@@ -1,4 +1,3 @@
-from . import user_database
 from . import settings
 import os
 import time
@@ -21,18 +20,4 @@ def clean_temp_folder(p_all_files = False):
             pass
 
 def startup_procedure():
-    user_database.work()
     clean_temp_folder(True)
-
-    #removing existing sessions
-    database_sessions = OmniDatabase.Generic.InstantiateDatabase(
-        'sqlite','','',settings.SESSION_DATABASE,'','','0',''
-    )
-    try:
-        database_sessions.v_connection.Execute('''
-            delete
-            from django_session
-        ''')
-    except Exception as exc:
-        print('Error:')
-        print(exc)
