@@ -12,6 +12,10 @@ import OmniDB_app.include.Spartacus.Utils as Utils
 from OmniDB_app.include.Session import Session
 from datetime import datetime
 
+from OmniDB_app.views.memory_objects import *
+
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_tree_info(request):
 
     v_return = {}
@@ -19,27 +23,10 @@ def get_tree_info(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_return['v_data'] = {
             'v_mode': 'database',
@@ -101,6 +88,8 @@ def get_tree_info(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_properties(request):
 
     v_return = {}
@@ -108,28 +97,11 @@ def get_properties(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_data = json_object['p_data']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_properties = []
     v_ddl = ''
 
@@ -150,6 +122,8 @@ def get_properties(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_tables(request):
 
     v_return = {}
@@ -157,28 +131,11 @@ def get_tables(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_tables = []
 
     try:
@@ -194,7 +151,8 @@ def get_tables(request):
                 'v_has_excludes': v_database.v_has_excludes,
                 'v_has_rules': v_database.v_has_rules,
                 'v_has_triggers': v_database.v_has_triggers,
-                'v_has_partitions': v_database.v_has_partitions
+                'v_has_partitions': v_database.v_has_partitions,
+                'v_has_statistics': v_database.v_has_statistics
             }
             v_list_tables.append(v_table_data)
     except Exception as exc:
@@ -206,6 +164,8 @@ def get_tables(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_columns(request):
 
     v_return = {}
@@ -213,29 +173,12 @@ def get_columns(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_columns = []
 
     try:
@@ -257,6 +200,8 @@ def get_columns(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_pk(request):
 
     v_return = {}
@@ -264,29 +209,12 @@ def get_pk(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_pk = []
 
     try:
@@ -304,6 +232,8 @@ def get_pk(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_pk_columns(request):
 
     v_return = {}
@@ -311,14 +241,6 @@ def get_pk_columns(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_pkey = json_object['p_key']
@@ -326,15 +248,6 @@ def get_pk_columns(request):
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_pk = []
 
     try:
@@ -352,6 +265,8 @@ def get_pk_columns(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_fks(request):
 
     v_return = {}
@@ -359,29 +274,12 @@ def get_fks(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_fk = []
 
     try:
@@ -402,6 +300,8 @@ def get_fks(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_fks_columns(request):
 
     v_return = {}
@@ -409,14 +309,6 @@ def get_fks_columns(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_fkey = json_object['p_fkey']
@@ -424,15 +316,6 @@ def get_fks_columns(request):
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_fk = []
 
     try:
@@ -454,6 +337,8 @@ def get_fks_columns(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_uniques(request):
 
     v_return = {}
@@ -461,29 +346,12 @@ def get_uniques(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_uniques = []
 
     try:
@@ -501,6 +369,8 @@ def get_uniques(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_uniques_columns(request):
 
     v_return = {}
@@ -508,14 +378,6 @@ def get_uniques_columns(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_unique = json_object['p_unique']
@@ -523,15 +385,6 @@ def get_uniques_columns(request):
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_uniques = []
 
     try:
@@ -549,6 +402,8 @@ def get_uniques_columns(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_indexes(request):
 
     v_return = {}
@@ -556,29 +411,12 @@ def get_indexes(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_indexes = []
 
     try:
@@ -597,6 +435,8 @@ def get_indexes(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_indexes_columns(request):
 
     v_return = {}
@@ -604,14 +444,6 @@ def get_indexes_columns(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_index = json_object['p_index']
@@ -619,15 +451,6 @@ def get_indexes_columns(request):
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_indexes = []
 
     try:
@@ -645,6 +468,8 @@ def get_indexes_columns(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_databases(request):
 
     v_return = {}
@@ -652,27 +477,10 @@ def get_databases(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_databases = []
 
     try:
@@ -691,6 +499,8 @@ def get_databases(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_roles(request):
 
     v_return = {}
@@ -698,27 +508,10 @@ def get_roles(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_roles = []
 
     try:
@@ -737,6 +530,8 @@ def get_roles(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_functions(request):
 
     v_return = {}
@@ -744,28 +539,11 @@ def get_functions(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_functions = []
 
     try:
@@ -785,6 +563,8 @@ def get_functions(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_function_fields(request):
 
     v_return = {}
@@ -792,29 +572,12 @@ def get_function_fields(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_function = json_object['p_function']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_fields = []
 
     try:
@@ -834,6 +597,8 @@ def get_function_fields(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_function_definition(request):
 
     v_return = {}
@@ -841,28 +606,11 @@ def get_function_definition(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_function = json_object['p_function']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_return['v_data'] = v_database.GetFunctionDefinition(v_function)
     except Exception as exc:
@@ -872,6 +620,8 @@ def get_function_definition(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_procedures(request):
 
     v_return = {}
@@ -879,28 +629,11 @@ def get_procedures(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_functions = []
 
     try:
@@ -920,6 +653,8 @@ def get_procedures(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_procedure_fields(request):
 
     v_return = {}
@@ -927,29 +662,12 @@ def get_procedure_fields(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_function = json_object['p_procedure']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_fields = []
 
     try:
@@ -969,6 +687,8 @@ def get_procedure_fields(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_procedure_definition(request):
 
     v_return = {}
@@ -976,28 +696,11 @@ def get_procedure_definition(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_function = json_object['p_procedure']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_return['v_data'] = v_database.GetProcedureDefinition(v_function)
     except Exception as exc:
@@ -1007,6 +710,8 @@ def get_procedure_definition(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_sequences(request):
 
     v_return = {}
@@ -1014,27 +719,12 @@ def get_sequences(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_schema = json_object['p_schema']
 
     v_database = v_session.v_databases[v_database_index]['database']
 
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_sequences = []
 
     try:
@@ -1053,6 +743,8 @@ def get_sequences(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_views(request):
 
     v_return = {}
@@ -1060,28 +752,11 @@ def get_views(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_tables = []
 
     try:
@@ -1101,6 +776,8 @@ def get_views(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_views_columns(request):
 
     v_return = {}
@@ -1108,29 +785,12 @@ def get_views_columns(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     v_list_columns = []
 
     try:
@@ -1151,6 +811,8 @@ def get_views_columns(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def get_view_definition(request):
 
     v_return = {}
@@ -1158,29 +820,12 @@ def get_view_definition(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_view = json_object['p_view']
     v_schema = json_object['p_schema']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_return['v_data'] = v_database.GetViewDefinition(v_view, v_schema)
     except Exception as exc:
@@ -1190,6 +835,8 @@ def get_view_definition(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def kill_backend(request):
 
     v_return = {}
@@ -1197,30 +844,13 @@ def kill_backend(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_pid            = json_object['p_pid']
     v_tab_id = json_object['p_tab_id']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
-        v_data = v_database.v_connection.Terminate(v_pid)
+        v_data = v_database.Terminate(v_pid)
     except Exception as exc:
         v_return['v_data'] = {'password_timeout': True, 'message': str(exc) }
         v_return['v_error'] = True
@@ -1228,6 +858,8 @@ def kill_backend(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def template_select(request):
 
     v_return = {}
@@ -1235,29 +867,12 @@ def template_select(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_tab_id = json_object['p_tab_id']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_template = v_database.TemplateSelect(v_schema, v_table).v_text
     except Exception as exc:
@@ -1271,6 +886,8 @@ def template_select(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def template_insert(request):
 
     v_return = {}
@@ -1278,29 +895,12 @@ def template_insert(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_tab_id = json_object['p_tab_id']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_template = v_database.TemplateInsert(v_schema, v_table).v_text
     except Exception as exc:
@@ -1314,6 +914,8 @@ def template_insert(request):
 
     return JsonResponse(v_return)
 
+@user_authenticated
+@database_required(p_check_timeout = True, p_open_connection = True)
 def template_update(request):
 
     v_return = {}
@@ -1321,29 +923,12 @@ def template_update(request):
     v_return['v_error'] = False
     v_return['v_error_id'] = -1
 
-    #Invalid session
-    if not request.session.get('omnidb_session'):
-        v_return['v_error'] = True
-        v_return['v_error_id'] = 1
-        return JsonResponse(v_return)
-
-    v_session = request.session.get('omnidb_session')
-
     json_object = json.loads(request.POST.get('data', None))
     v_database_index = json_object['p_database_index']
     v_tab_id = json_object['p_tab_id']
     v_table = json_object['p_table']
     v_schema = json_object['p_schema']
 
-    v_database = v_session.v_tab_connections[v_tab_id]
-
-    #Check database prompt timeout
-    v_timeout = v_session.DatabaseReachPasswordTimeout(int(v_database_index))
-    if v_timeout['timeout']:
-        v_return['v_data'] = {'password_timeout': True, 'message': v_timeout['message'] }
-        v_return['v_error'] = True
-        return JsonResponse(v_return)
-
     try:
         v_template = v_database.TemplateUpdate(v_schema, v_table).v_text
     except Exception as exc:
